A unix domain sockets library that supports abstract addresses, fd-passing, SOCK_SEQPACKET sockets and more.
File-descriptor passing and abstract socket support for stream and datagram sockets is provided via extension traits for existing types.
Also it support UnixStream for Windows i.e AF_INET SOCK_STREAM with some limitations.
See README for status of operating system support and other general info.
In seqpacket.rs a UnixSeqpacketConn:
If the feature unsatable_preview is enabled, a send_vectored_with_ancillary_to,
send_vectored_with_ancillary, recv_vectored_with_ancillary_from,
recv_vectored_with_ancillary are enabled.
A legacy (method provided not by std but this crate) is recv_vectored_ancillary.
Re-exports
pub extern crate xio_rs;
Modules
Structs
- Ancillary
- An iterator over ancillary messages received with
recv_ancillary(). - Ancillary
Buf - An ancillary data buffer that supports any capacity.
- Unix
Seqpacket Conn - An unix domain sequential packet connection.
- Unix
Seqpacket Listener - An unix domain listener for sequential packet connections.
- Unix
Socket Addr - A unix domain socket address.
Enums
- Addr
Name - An enum representation of an unix socket address.
- Ancillary
Item - One ancillary message produced by
Ancillary - Conn
Credentials - Credentials of the peer process when it called
connect(),accept()orpair().
Traits
- Unix
Datagram Ext - Extension trait for
std::os::unix::net::UnixDatagramand nonblocking equivalents. - Unix
Listener Ext - Extension trait for using
UnixSocketAddrwithUnixListenertypes. - Unix
Stream Ext - Extension trait for
std::os::unix::net::UnixStreamand nonblocking equivalents.