WindowsUnixStream

Struct WindowsUnixStream 

pub struct WindowsUnixStream { /* private fields */ }
An unix domain stream packet connection for Windows.

It requires WSA version 2.0 or 2.2 i.e Windows 10 and above.

The crate requests version 2.2 by default!

Is simular to the UnixStream but with some limitations:

  • no SOCK_DGRAM or SOCK_SEQPACKET support

  • Ancillary data like SCM_RIGHTS SCM_CREDENTIALS

  • Autobind feature

  • socketpair

Examples

let path = "server3.sock";
 
let client = WindowsUnixStream::connect(path).unwrap();
client.send(b"first").unwrap();
client.send(b"second").unwrap();

Implementations

Source

impl WindowsUnixStream

pub fn connect<P: AsRef<Path>>(path: P) -> Result<Self, Error>

Connects to an unix stream server listening at path.

This is a wrapper around connect_unix_addr() for convenience and compatibility with std.

pub fn connect_unix_addr(addr: &UnixSocketAddr) -> Result<Self, Error>

Connects to an unix seqpacket server listening at addr.

pub fn connect_from_to_unix_addr( from: &UnixSocketAddr, to: &UnixSocketAddr, ) -> Result<Self, Error>

Binds to an address before connecting to a listening seqpacet socket.

pub fn pair() -> Result<(Self, Self), Error>

Creates a socket pair.

pub fn try_clone(&self) -> Result<Self>

pub fn set_nonblocking(&self, nonblk: bool) -> Result<()>

pub fn set_no_inherit(&self, no_inh: bool) -> Result<()>

Cloexec

pub fn set_write_timeout(&self, timeout: Option<Duration>) -> Result<()>

pub fn write_timeout(&self) -> Result<Option<Duration>>

pub fn set_read_timeout(&self, timeout: Option<Duration>) -> Result<()>

pub fn read_timeout(&self) -> Result<Option<Duration>>

pub fn local_unix_addr(&self) -> Result<UnixSocketAddr, Error>

Returns the address of this side of the connection.

pub fn peer_unix_addr(&self) -> Result<UnixSocketAddr, Error>

Returns the address of the other side of the connection.

pub fn send(&self, packet: &[u8]) -> Result<usize, Error>

Sends a packet to the peer.

pub fn recv(&self, buffer: &mut [u8]) -> Result<usize, Error>

Receives a packet from the peer.

pub fn recv_vectored( &self, bufs: &mut [IoSliceMut<'_>], ) -> Result<(usize, RecvFlags)>

pub fn send_vectored(&self, bufs: Vec<IoSlice<'_>>) -> Result<usize>

Windows consumes the io::IoSlice instances, so can no logner be accessed. The io::Write borrows, but it is not correct. DO not access the slices after it were sent.

pub fn take_error(&self) -> Result<Option<Error>, Error>

Returns the value of the SO_ERROR option.

This might only provide errors generated from nonblocking connect()s, which this library doesn’t support. It is therefore unlikely to be useful, but is provided for parity with stream counterpart in std.

pub fn shutdown(&self, how: Shutdown) -> Result<()>

Allows to shudown receiving/sending or both sides.