AncillaryItem

Enum AncillaryItem 

pub enum AncillaryItem {
    Fds(Vec<OwnedFd>),
    Credentials(ReceivedCredentials),
    Unsupported,
}
One ancillary message produced by Ancillary

Variants



Fds(Vec<OwnedFd>)

One or more file descriptors sent by the peer.

Consumer of the iterator is responsible for closing them.



Credentials(ReceivedCredentials)

Credentials of the sending process.



Unsupported

An unknown or unsupported ancillary message type was received.

It’s up to you whether to ignore or treat as an error.