pub struct WindowsUnixStream { /* private fields */ }Expand description
An unix domain stream packet connection for Windows.
It requires WSA version 2.0 or 2.2 i.e Windows 10 and above.
The crate requests version 2.2 by default!
Is simular to the UnixStream but with some limitations:
-
no
SOCK_DGRAMorSOCK_SEQPACKETsupport -
Ancillary data like
SCM_RIGHTSSCM_CREDENTIALS -
Autobind feature
-
socketpair
Examples
let path = "server3.sock";
let client = WindowsUnixStream::connect(path).unwrap();
client.send(b"first").unwrap();
client.send(b"second").unwrap();Implementations
Sourceimpl WindowsUnixStream
impl WindowsUnixStream
Sourcepub fn connect<P: AsRef<Path>>(path: P) -> Result<Self, Error>
pub fn connect<P: AsRef<Path>>(path: P) -> Result<Self, Error>
Connects to an unix stream server listening at path.
This is a wrapper around connect_unix_addr()
for convenience and compatibility with std.
Sourcepub fn connect_unix_addr(addr: &UnixSocketAddr) -> Result<Self, Error>
pub fn connect_unix_addr(addr: &UnixSocketAddr) -> Result<Self, Error>
Connects to an unix seqpacket server listening at addr.
Sourcepub fn connect_from_to_unix_addr(
from: &UnixSocketAddr,
to: &UnixSocketAddr,
) -> Result<Self, Error>
pub fn connect_from_to_unix_addr( from: &UnixSocketAddr, to: &UnixSocketAddr, ) -> Result<Self, Error>
Binds to an address before connecting to a listening seqpacet socket.
pub fn try_clone(&self) -> Result<Self>
pub fn set_nonblocking(&self, nonblk: bool) -> Result<()>
Sourcepub fn set_no_inherit(&self, no_inh: bool) -> Result<()>
pub fn set_no_inherit(&self, no_inh: bool) -> Result<()>
Cloexec
pub fn set_write_timeout(&self, timeout: Option<Duration>) -> Result<()>
pub fn write_timeout(&self) -> Result<Option<Duration>>
pub fn set_read_timeout(&self, timeout: Option<Duration>) -> Result<()>
pub fn read_timeout(&self) -> Result<Option<Duration>>
Sourcepub fn local_unix_addr(&self) -> Result<UnixSocketAddr, Error>
pub fn local_unix_addr(&self) -> Result<UnixSocketAddr, Error>
Returns the address of this side of the connection.
Sourcepub fn peer_unix_addr(&self) -> Result<UnixSocketAddr, Error>
pub fn peer_unix_addr(&self) -> Result<UnixSocketAddr, Error>
Returns the address of the other side of the connection.
pub fn recv_vectored( &self, bufs: &mut [IoSliceMut<'_>], ) -> Result<(usize, RecvFlags)>
Sourcepub fn send_vectored(&self, bufs: Vec<IoSlice<'_>>) -> Result<usize>
pub fn send_vectored(&self, bufs: Vec<IoSlice<'_>>) -> Result<usize>
Windows consumes the io::IoSlice instances, so can no logner be accessed. The io::Write borrows, but it is not correct. DO not access the slices after it were sent.
Sourcepub fn take_error(&self) -> Result<Option<Error>, Error>
pub fn take_error(&self) -> Result<Option<Error>, Error>
Returns the value of the SO_ERROR option.
This might only provide errors generated from nonblocking connect()s,
which this library doesn’t support. It is therefore unlikely to be
useful, but is provided for parity with stream counterpart in std.